
/*
 * XKOBO, a video-oriented game
 * Copyright (C) 1995,1996  Akira Higuchi
 *     a-higuti@math.hokudai.ac.jp
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */
#include <stdio.h>
#include <unistd.h>
#include "pspctrl.h"

#include "manage.h"
#include "xkobo.h"
#include "key.h"

int _key::space;
int _key::left;
int _key::up;
int _key::down;
int _key::right;
int _key::dl;
int _key::dr;
int _key::ul;
int _key::ur;
int _key::shot;
int _key::direction;
bool _key::vStart_Depressed = false;
bool _key::vLT_Depressed = false;
bool _key::vRT_Depressed = false;
bool _key::vSelect_Depressed = false;
bool _key::vAuto_Fire = false;

int vAction_Buttons = (PSP_CTRL_SQUARE | PSP_CTRL_CIRCLE | PSP_CTRL_TRIANGLE | PSP_CTRL_CROSS);

void _key::init()
{
    direction = 1;
    clear();
    sceCtrlSetSamplingCycle(0);
    sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);
}

void _key::clear()
{
    left = 0; right = 0; up = 0; down = 0; shot = 0;
}

void _key::poll() {
	bool fChanged = false;
	SceCtrlData ctrl;
	sceCtrlPeekBufferPositive(&ctrl, 1);
	clear();

	if(ctrl.Lx < 64) {left = 1; fChanged = true;}
	if(ctrl.Lx > 192) {right = 1; fChanged = true;}
	if(ctrl.Ly < 64) {up = 1; fChanged = true;}
	if(ctrl.Ly > 192) {down = 1; fChanged = true;}
	if(vAuto_Fire && manage.state == playing){shot = 1; fChanged = true;}

	if(ctrl.Buttons != 0){ // This saves us a little processing power.
		if(manage.state == playing){
			if(ctrl.Buttons & PSP_CTRL_LEFT) {left = 1; fChanged = true;}
			if(ctrl.Buttons & PSP_CTRL_RIGHT) {right = 1; fChanged = true;}
			if(ctrl.Buttons & PSP_CTRL_UP) {up = 1; fChanged = true;}
			if(ctrl.Buttons & PSP_CTRL_DOWN) {down = 1; fChanged = true;}
			if(ctrl.Buttons & vAction_Buttons){shot = 1; fChanged = true;}
		}else if(manage.state == title){
			if(ctrl.Buttons & PSP_CTRL_LTRIGGER){
				if(!vLT_Depressed){
					manage.minus_key();
					vLT_Depressed = true;
				}
			}else vLT_Depressed = false;

			if(ctrl.Buttons & PSP_CTRL_RTRIGGER){
				if(!vRT_Depressed){
					manage.plus_key();
					vRT_Depressed = true;
				}
			}else vRT_Depressed = false;
		}

		if(ctrl.Buttons & PSP_CTRL_START){
			if(!vStart_Depressed){
				manage.start_key();
				vStart_Depressed = true;
				if(manage.state == paused) return;
			}
		}else vStart_Depressed = false;

		if(ctrl.Buttons & PSP_CTRL_SELECT){
			if(!vSelect_Depressed){
				vAuto_Fire = !vAuto_Fire;
				vSelect_Depressed = true;
			}
		}else vSelect_Depressed = false;

	}else{
  		vStart_Depressed = false; vLT_Depressed = false; vRT_Depressed = false;
  		vSelect_Depressed = false;
	}
	if(fChanged) change();
}

void _key::change_up(unsigned long r)
{
    up = (r == 0) ? 1 : 0;
    _key::change();
}

void _key::change_down(unsigned long r)
{
    down = (r == 0) ? 1 : 0;
    _key::change();
}

void _key::change_left(unsigned long r)
{
    left = (r == 0) ? 1 : 0;
    _key::change();
}

void _key::change_right(unsigned long r)
{
    right = (r == 0) ? 1 : 0;
    _key::change();
}

void _key::change_shot(unsigned long r)
{
    shot = (r == 0) ? 1 : 0;
    _key::change();
}

void _key::change()
{
	int lr = left - right + ul - ur + dl - dr;
    int ud = up   - down  + ul + ur - dl - dr;
    if (lr > 0){
        if (ud > 0) direction = 8;
        else if (ud < 0) direction = 6;
        else direction = 7;
    }
    else if (lr < 0){
        if (ud > 0) direction = 2;
        else if (ud < 0) direction = 4;
        else direction = 3;
    }
    else {
        if (ud > 0) direction = 1;
        else if (ud < 0) direction = 5;
    }
}
