
/*
 * XKOBO, a video-oriented game
 * Copyright (C) 1995,1996  Akira Higuchi
 *     a-higuti@math.hokudai.ac.jp
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <pspdisplay.h>
#include <psppower.h>
#include <pspkernel.h>
#include <pspdebug.h>
#include <pspmoduleinfo.h>

#include "xkobo.h"
#include "config.h"
#include "tiff.h"
#include "screen.h"
#include "manage.h"
#include "key.h"
#include "random.h"
#include "version.h"
#include "renderer/control.h"
#include "renderer/raster.h"

win_cmap        wbase;
win_chip        wchip;
win_backing     wradar;
win_backing     wscore;
int             scale_log2;
int             mouse_x = 0;
int             mouse_y = 0;
int             cheat_mode;

PSP_MODULE_INFO("PSPKOBO", 0, 1, 1);
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER);

/* Exit callback */
int exit_callback(int arg1, int arg2, void *common) {
	sceKernelExitGame();
	return 0;
}

/* Callback thread */
int CallbackThread(SceSize args, void *argp) {
	int cbid;

	cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
	sceKernelRegisterExitCallback(cbid);

	sceKernelSleepThreadCB();

	return 0;
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void) {
	int thid = 0;

	thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, 0, 0);
	if(thid >= 0) {
		sceKernelStartThread(thid, 0, 0);
	}
	return thid;
}

int main()
{
	int sprite_policy = 1;
	int cmap_policy = 1;
	scale_log2 = 0;
	cheat_mode = 0;

	SetupCallbacks();
	rast_init();

	wbase.make((win *)NULL, 0, 0,
			   MARGIN * 3 + ((WSIZE + MAP_SIZEX) << scale_log2),
			   MARGIN * 2 + (WSIZE << scale_log2), cmap_policy);
	wchip.make(&wbase, MARGIN, MARGIN,
			   WSIZE << scale_log2, WSIZE << scale_log2,
			   (SCREEN_SIZEX + WSIZE) << scale_log2,
			   (SCREEN_SIZEY + WSIZE) << scale_log2,
			   320 << scale_log2, 240 << scale_log2, sprite_policy);
	wradar.make(&wbase,WSIZE << scale_log2, ((WSIZE - MAP_SIZEY) << scale_log2),
				MAP_SIZEX << 1, MAP_SIZEY << 1);

	wscore.make((win *)NULL, 0, 0, WSCORE_SIZEX, WSCORE_SIZEY);

	link_tiff(NULL, wbase, wchip);

	wbase.foreground(0xffff);
	wbase.background(0);
	wbase.clear();
	wchip.foreground(0xffff);
	wchip.background(0);
	wradar.foreground(0xffff);
	wradar.background(0);
	wradar.setborder(0);
	wradar.endx = 272;
	wradar.endy = 0;
	wscore.foreground(0xffff);
	wscore.background(0);
	wscore.clear();
	wscore.endx = SCORE_X;
	wscore.endy = SCORE_Y;

	rand_num.init();
	manage.init();

	while(true){
		if (manage.mainloop()) break;
		sceDisplayWaitVblankStart(); // This controls the timing by waiting for the end of the video refresh (i.e.: the vertical blanking interval)
									// before allowing the mainloop to run again. This means a max of 60fps for the PSP.
	}

	sceKernelSleepThread();
	return 0;
}


